/*
 * Filename: macaddr.cmd
 *   Author: JAN-ERIK
 *  Created: Mon Sep 10 2012
 *  Purpose: Retrieve MAC Address
 *  Changes: 2019-07-25, Added support for Router
 *           2019-09-14, Added support for localhost
 *           2019-09-15, OS2ENVIRONMENT -> ENVIRONMENT
 *           2019-10-11, Rewrite to use TCPIP HOST command
 *           2025-03-15, Traverse ip-numbers in subnet, use environment variables to control output
 *  Updater: REMY-DODIN  
 *  Changes: 2025-03-27, Added SockGH, Anetstat, MissingDomain procedures. Updated rxGetMacAddr procedure. Reduced output messages. Added initial Computer host(name,ip) 
 *  Changes: 2025-03-28, Added UNDEFINED hosname option, added @ip into WPS parms as second parm. For UNDEFINED equipment, @ip is used for some cmds 
 *  			 Replaced UNDEFINED with UHOST-xxx with xxx equal latest @ip qualifier      
 *  			 Restored important unwanted deleted line.       
 *  Changes: 2025-03-29, Added correction for malformed @mac e.g. NN:N :NN:NN:NN:NN or N :NN:NN:NN:NN:N. Reduced process time
 *                       Added a WPS object to allow full network devices scan/status update following an arp flush. Created when run with FLC=1
 *  Changes: 2025-03-31, Added gateway and uc indication. Corrected Tempip var, updated a few codes
 *                       macaddr.cfg generated at FLC=1 run time including " @mac @ip hostname " with hostname been user modified
 *  Changes: 2025-04-01, A few minor changes   		        
 *  Changes: 2025-04-03, Added interface number on Domain folder. Updated missing domain procedure to identify any @ip missing domain name and not only UC/GW  
 *  Changes: 2025-04-04, Added a status of (removed) when an already existing network host is removed from the network. If the removed host is added using same hostname
 *			 and @ip address as before, its status goes up with included @mac else a new icon should be created with new @ip, hostname, @mac     
 *  Changes: 2025-04-10, Corrected a typo preventing correct cfg generated file with flush. Added a retention day to remove WPS object after n days (removed) status 
 */

/* Use following lines if WOL and/or NET not set into config.sys - Value can be 0 or 1 */ 
/* 'SET WOL=1' */
/* 'SET NET=1' */
/* 'SET FLC=1' */
/* 'SET MVN=0' */

/* Adjust WPS destroy object retention days for removed hosts - default value 7 days or 0 to "disable" */ 
/* retdays = 0 */

/* clean screen */
'@cls'

/* special trace function follows */
/* trace all */

PARSE SOURCE . . this
thispath = FILESPEC( 'd', this )||FILESPEC( 'p', this )
stem = 'host.'
UseIntrfAsDomain = ''
LANPC=0
ThisHIP=''
GWip = ''
macfile = thispath||'macaddr.cfg'
macfilestring = ''
Intf_NETX = ''
cfg_found = ''
hpos = 4
new_input = 0
Rrc = 0

    CALL RxFuncAdd 'SockLoadFuncs', 'rxsock', 'SockLoadFuncs';
    CALL SockLoadFuncs( '*' );

If VALUE( 'MVN',, 'OS2ENVIRONMENT' ) = 0 THEN Say "Scan started using env parameters: WOL("||VALUE( 'WOL',, 'OS2ENVIRONMENT' )||") and NET("||VALUE( 'NET',, 'OS2ENVIRONMENT' )||")"  

'@NETSTAT -p 2>&1 |RXQUEUE'
maxqueued = queued()
DO queued()
	Parse pull nentries
		If pos('.',nentries)>0 & pos(':',nentries)>0 THEN
			DO  		
				LANPC = 2
				leave
			END
		If maxqueued = 5 & pos('.',nentries)>0 & pos(':',nentries)=0 then LANPC = 1
END
DO WHILE QUEUED() > 0; PARSE PULL; END;
IF LANPC < 2 THEN CALL LANKO 

CALL MAC_READ

IF 0 < ARG() THEN
    count = rxGetMacAddr( ARG(1) )
ELSE
    count = rxGetMacAddr()

DO i = 1 TO count
    /*magic.cmd*/
    /*Wake on Lan in REXX*/
    /*MAC = '11:22:33:44:55:66';*/
    MAC = host.i.macaddr
    IF ( VALUE( 'WOL',, 'OS2ENVIRONMENT' ) = 1 ) & ( translate(host.i.macaddr) \= '(INCOMPLETE)' ) & ( translate(host.i.macaddr) \= 'RESOLVE' ) THEN
    DO
        X = TRANSLATE( 'abcdefghijkl', host.i.macaddr, 'ab:cd:ef:gh:ij:kl' )

	If host.i.name = '' THEN
		DO
			TempIp = host.i.ipaddr
			TempHName = host.i.name
			Call SockGH
			host.i.name = TempHName
			host.i.ipaddr = TempIp
		END
        If VALUE( 'MVN',, 'OS2ENVIRONMENT' ) \= 1 THEN SAY host.i.name||':' host.i.ipaddr||':' MAC;
	SCT = SockSocket( 'AF_INET', 'SOCK_DGRAM', 'IPPROTO_UDP' );
        RC = SockSetSockOpt( SCT, 'SOL_SOCKET', 'SO_BROADCAST', '1' );
        RC = SockIOCtl( SCT, 'FIONBIO', '0' );
        adr.!family = 'AF_INET';
        /*adr.!addr   = '255.255.255.255';*/
        adr.!addr   = host.i.ipaddr
        adr.!port   = 9;
        S = COPIES( D2C(255), 6 );
        T = COPIES( X, 16);
        G = COPIES( D2C(0), 6 );
        RC = SockConnect( SCT, "adr.!" );
        STG = S||T||G;
        If VALUE( 'MVN',, 'OS2ENVIRONMENT' ) \= 1 THEN SAY 'wake on lan sent: ' left(STG,50)||' ...'
        RC = SockSend( SCT, STG );
        CALL SockSoClose SCT; 
    END
    ELSE 
	DO
		If VALUE( 'MVN',, 'OS2ENVIRONMENT' ) \= 1 THEN SAY host.i.name||':' host.i.ipaddr||':' MAC;
	END
    IF VALUE( 'NET',, 'OS2ENVIRONMENT' ) = 1 THEN
    DO
        /* Load RexxUtil Library */
        IF RxFuncQuery( 'SysLoadFuncs' ) THEN
        DO
            CALL RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
            CALL SysLoadFuncs
        END
        IF SysCreateObject( 'WPFolder', 'Network Map', '<WP_CONNECTIONSFOLDER>', 'OBJECTID=<WP_NETMAP>;ICONFILE='||thispath||'Navigation - Folder.ICO;ICONNFILE=1,'||thispath||'Navigation - Open Folder.ICO;', 'U' ) Then
        DO
            PARSE VALUE host.i.name WITH hosted'.'domain
            IF domain = '' THEN
                domain = 'Workgroup'
	    IF host.i.ipaddr =  ThisHIP then hosted=hosted||'(UC)'
	    IF host.i.ipaddr =  GWip then hosted=hosted||'(gateway)'
	    INTFDomain = ''	
	    IF wordpos(substr(host.i.ipaddr,1,lastpos('.',host.i.ipaddr)-1),Intf_NETX) > 0 THEN INTFDomain = word( Intf_NETX, wordpos(substr(host.i.ipaddr,1,lastpos('.',host.i.ipaddr)-1),Intf_NETX)+1)		
	    ELSE
		DO
		    IF wordpos(substr(host.i.ipaddr,1,pos('.',host.i.ipaddr,pos('.',host.i.ipaddr,1)+1)-1),Intf_NETX) > 0 THEN INTFDomain = word( Intf_NETX, wordpos(substr(host.i.ipaddr,1,pos('.',host.i.ipaddr,pos('.',host.i.ipaddr,1)+1)-1),Intf_NETX)+1)		
		END 	
            IF SysCreateObject( 'WPFolder', domain '['INTFDomain']', '<WP_NETMAP>', 'OBJECTID=<WP_'||TRANSLATE( domain )||INTFDomain||'>;ICONFILE='||thispath||'Network - Server VI.ICO;ICONNFILE=1,'||thispath||'Network - Server VI.ICO;', 'U' ) Then
            DO
                IF host.i.macaddr = '(incomplete)' THEN
                DO
                    IF host.i.macaddr = '(incomplete)' then host.i.macaddr = '(Offline)'
                    ico = 'Network - Connect.ICO'
                    END
                ELSE
                    ico = 'Hardware - Terminal Mini.ICO'
		in_cfg = 0 ; k = 0
		IF cfg.0 > 0 THEN
			DO k = 1 for cfg.0
				IF ( pos(host.i.macaddr, cfg.k,1) > 0 & host.i.macaddr \= '(Offline)' ) | ( host.i.macaddr = '(Offline)' & pos(host.i.ipaddr, cfg.k,1) > 0 ) | ( pos(host.i.ipaddr, cfg.k,1) > 0 & host.i.macaddr \= '(Offline)' & pos(word(translate(host.i.name,' ','.'),1), cfg.k,1) > 0 ) THEN
					DO
						hosted = subword(cfg.k,hpos)
						IF pos('.',hosted,1) > 0 THEN hosted = substr(hosted,1,pos('.',hosted,1)-1)
						cfg_found = strip(cfg_found||' '||k)
						If host.i.macaddr \= '(Offline)' THEN cfg.k = date(B)||' '||subword(cfg.k,2)
						in_cfg = k
					END
 			END
		IF 0 = ARG() THEN
			DO 
				IF in_cfg = '0' THEN DO				     	
				    macfilestring=Strip(macfilestring||date(B)||' '||host.i.macaddr||' '||host.i.ipaddr||' '||hosted||'.'||domain||'.'||INTFDomain||x2c("0D0A"))
				    new_input = 1	
				END
				ELSE macfilestring=Strip(macfilestring||cfg.in_cfg||x2c("0D0A"))
				
			END 
                CALL SysCreateObject 'WPProgram', STRIP( hosted )||'^'||host.i.macaddr, '<WP_'||TRANSLATE( domain )||INTFDomain||'>', 'OBJECTID=<WP_'||TRANSLATE( STRIP( hosted ) )||'>;TITLE='||STRIP( hosted )||'^'||host.i.macaddr||';EXENAME='||this||';PARAMETERS='||host.i.name||' '||host.i.ipaddr';PROGTYPE=PROG_WINDOWABLEVIO;MINIMIZED=YES;ICONFILE='||thispath||ico||';SET WOL=1;SET NET=1', 'U'
            END
	    If VALUE( 'MVN',, 'OS2ENVIRONMENT' ) \= 1 THEN SAY '> WPS Refreshed for: '||host.i.name host.i.ipaddr
        END
    END
    IF words(cfg_found) > 0 & 0 = ARG() THEN
	DO
		DO k = 1 for cfg.0
			IF wordpos(k,cfg_found)=0 THEN
			DO
				hosted = subword(cfg.k,hpos)
				IF pos('.',hosted,1) > 0 THEN
					DO
						domain = substr(hosted,pos('.',hosted,1)+1)
						IF pos('.',domain) > 0 THEN 
							DO
								INTFDomain = substr(domain,pos('.',domain,1)+1)
								domain = substr(domain,1,pos('.',domain,1)-1)
							END
						hosted = substr(hosted,1,pos('.',hosted,1)-1)						
					END   
				host.kmacaddr = '(Removed)'
				host.kipaddr = word(cfg.k,hpos-1)
				ico = 'Network - Connect.ICO'
				If datatype(word(cfg.k,1),'N') = 1 THEN
					DO
						IF datatype(retdays,'N') = 0 THEN retdays = 7  
 						IF word(cfg.k,1) + retdays < date(B) & retdays \= 0 THEN Call SysDestroyObject '<WP_'||TRANSLATE( STRIP( hosted ) )||'>'
						ELSE Call SysCreateObject 'WPProgram', STRIP( hosted )||'^'||host.kmacaddr, '<WP_'||TRANSLATE( domain )||INTFDomain||'>', 'OBJECTID=<WP_'||TRANSLATE( STRIP( hosted ) )||'>;TITLE='||STRIP( hosted )||'^'||host.kmacaddr||';EXENAME='||this||';PARAMETERS='||hosted||' '||host.kipaddr';PROGTYPE=PROG_WINDOWABLEVIO;MINIMIZED=YES;ICONFILE='||thispath||ico||';SET WOL=1;SET NET=1', 'U'
					END
				ELSE Call SysCreateObject 'WPProgram', STRIP( hosted )||'^'||host.kmacaddr, '<WP_'||TRANSLATE( domain )||INTFDomain||'>', 'OBJECTID=<WP_'||TRANSLATE( STRIP( hosted ) )||'>;TITLE='||STRIP( hosted )||'^'||host.kmacaddr||';EXENAME='||this||';PARAMETERS='||hosted||' '||host.kipaddr';PROGTYPE=PROG_WINDOWABLEVIO;MINIMIZED=YES;ICONFILE='||thispath||ico||';SET WOL=1;SET NET=1', 'U'
				IF i = count THEN macfilestring=Strip(macfilestring||cfg.k||x2c("0D0A"))
			END 
		END
	END
    IF VALUE( 'FLC',, 'OS2ENVIRONMENT' ) = 1 & ARG() = 0 THEN 
	DO
		CALL SysCreateObject 'WPProgram', STRIP( 'ARP-F UPDT' )||'^'||'Network hosts', '<WP_NETMAP>', 'OBJECTID=<WP_'||TRANSLATE( STRIP( 'FULLREFRESH' ) )||'>;TITLE='||STRIP( 'ARP-F UPDT' )||'^'||'Network hosts'||';EXENAME='||this||';PROGTYPE=PROG_WINDOWABLEVIO;MINIMIZED=YES;ICONFILE='||thispath||ico||';SET WOL=1;SET NET=1;SET FLC=1;SET MVN=1', 'U'
		CALL SysCreateObject 'WPProgram', STRIP( 'REFRESH' )||'^'||'Network hosts', '<WP_NETMAP>', 'OBJECTID=<WP_'||TRANSLATE( STRIP( 'REFRESHONLY' ) )||'>;TITLE='||STRIP( 'REFRESH' )||'^'||'Network hosts'||';EXENAME='||this||';PROGTYPE=PROG_WINDOWABLEVIO;MINIMIZED=YES;ICONFILE='||thispath||ico||';SET WOL=1;SET NET=1;SET MVN=1', 'U'
	END
END
IF VALUE( 'FLC',, 'OS2ENVIRONMENT' ) = 1 | new_input = 1 | words(cfg_found) > 0 THEN CALL MAC_WRITE 
RETURN 0

rxGetMacAddr: PROCEDURE EXPOSE (stem) UseIntrfAsDomain ThisHIP GWip Intf_NETX /*<COMPUTERNAME>*/
    isStem = ( RIGHT( stem, 1 )  = '.' )
    stemval = STRIP( stem, 'T', '.' )
    retval = 0
    i = 0
/* No parameters given, list all computers */
    IF ARG() = 0 THEN
    DO
	IF VALUE( 'FLC',, 'OS2ENVIRONMENT' ) = 1 THEN '@ARP -f 2>&1|RXQUEUE'
        DO WHILE QUEUED() > 0; PARSE PULL; END;
        IF VALUE( 'ACTIVE',, 'OS2ENVIRONMENT' ) = 1 THEN
        DO
            '@PING 255.255.255.255 56 1 2>&1|RXQUEUE'
            DO WHILE QUEUED() > 0
                PARSE VALUE REVERSE( LINEIN( 'QUEUE:' ) ) WITH ' :'ipnumber.i' '.
                ipnumber.i = STRIP( REVERSE( ipnumber.i ) )
                IF IsIP( ipnumber.i ) THEN
                    i = i + 1
            END
        END
        ELSE i = 1
        ipnumber.0 = i
        hostname.0 = i
	lcounter = i
	
        '@HOST' VALUE( 'HOSTNAME',, 'ENVIRONMENT' ) '2>&1|RXQUEUE'
        PARSE VALUE LINEIN( 'QUEUE:' ) WITH hostname.i' = 'ipnumber.i

	/* used during test * ipnumber.i = '';hostname.i = VALUE( 'HOSTNAME',, 'ENVIRONMENT' ) */

	IF pos('unknown',hostname.i)>0 then hostname.i = VALUE( 'HOSTNAME',, 'ENVIRONMENT' )	 
	IF hostname.i = '' | ipnumber.i = '' then CALL Anetstat
	IF pos('.',hostname.i)=0 then 
		DO
			TempIp = ipnumber.i
			TempHName = hostname.i
			If pos('.',TempHname)=0 & ipnumber.i \= '' then Call MissingDomain
			hostname.i = TempHName
			ipnumber.i = TempIp
			If VALUE( 'MVN',, 'OS2ENVIRONMENT' ) \= 1 THEN SAY 'Computer (from sockets): ' hostname.i '  ' ipnumber.i
		END
	ELSE 
		DO
			TempIp = ipnumber.i
			TempHName = word(translate(hostname.i,' ','.'),1)
			If pos('.',TempHname) = 0 & ipnumber.i \= '' then Call MissingDomain
			ipnumber.i = TempIp
			IF pos('.',hostname.i) = 0 then hostname.i = TempHName	
			If VALUE( 'MVN',, 'OS2ENVIRONMENT' ) \= 1 THEN SAY 'Computer (from host): ' hostname.i '  ' ipnumber.i 
		END

	/* END */

        last = STRIP( ipnumber.i )
        j = 1
        IF VALUE( 'ACTIVE',, 'OS2ENVIRONMENT' ) = 1 THEN
        DO i = 1 TO ipnumber.0
            '@HOST' STRIP( ipnumber.i ) '2>&1|RXQUEUE'
            IF 0 < QUEUED() THEN
                PARSE VALUE LINEIN( 'QUEUE:' ) WITH ipnumber.j' = 'hostname.j
            IF 0 < LENGTH( hostname.j ) THEN
		DO
			TempIp = ipnumber.j
			TempHName = word(translate(hostname.j,' ','.'),1)
			If pos('.',TempHname) = 0 & ipnumber.j \= '' then Call MissingDomain
			ipnumber.j = TempIp
			IF pos('.',hostname.j) = 0 then hostname.j = TempHName	
	               	j = j + 1
		END
	    ELSE 
		DO
			TempHName = ''
			TempIp = ipnumber.i 	
			Call SockGH
			ipnumber.j = TempIp
			hostname.j = TempHName	
			IF 0 < LENGTH( hostname.j ) THEN
		               	j = j + 1
		END
        END
        ELSE DO
            PARSE VALUE last WITH major'.'minor'.'subnet'.'this
            IF DATATYPE( this, 'W' ) THEN
            DO i = 1 TO 255
                '@HOST' major||'.'||minor||'.'||subnet||'.'||i '2>&1|RXQUEUE'
                IF 0 < QUEUED() THEN
			DO
			    QLine = LINEIN( 'QUEUE:' )
	                    IF pos('resolve',QLine) = 0 THEN PARSE VALUE QLine WITH ipnumber.j' = 'hostname.j
			    ELSE hostname.j = ''
			END
                IF 0 < LENGTH( hostname.j ) THEN
			DO
				TempIp = ipnumber.j
				TempHName = word(translate(hostname.j,' ','.'),1)
				IF pos('.',TempHname)=0 & ipnumber.j \= '' then Call MissingDomain
				ipnumber.j = TempIp
				IF pos('.',hostname.j) = 0 then hostname.j = TempHName	
		               	j = j + 1
			END
	        ELSE 
	            DO
			TempHName = ''
			TempIp = major||'.'||minor||'.'||subnet||'.'||i 	
			IF pos('resolve',QLine) = 0 THEN Call SockGH
			ipnumber.j = TempIp
			hostname.j = TempHName	
			IF 0 < LENGTH( hostname.j ) THEN
		               	j = j + 1
		    END    	 
            END
        END
        hostname.0 = j - 1

    END
    ELSE DO
        /* Search for a specific computer */
        hostname.0 = 1
        hostname.1 = word(ARG(1),1)
	ipnumber.0 = 0
	ipnumber.= ''
	IF 0 < ARG() & words(ARG(1))>1 THEN
		DO
			ipnumber.0 = 1
			ipnumber.1 = word(ARG(1),2)
		END
    END
    j = 0
    DO WHILE QUEUED() > 0; PARSE PULL; END;
    DO i = 1 TO hostname.0
	SWAPHIP = IsIP(hostname.i)
        If SWAPHIP = 1 & hostname.0 = 1 THEN DO
		ipnumber.i = hostname.i
		hostname.i = ''
		ipnumber.0 = 1
	END
        IF ipnumber.i = last THEN
            '@NETSTAT -n 2>&1|RXQUEUE'
        ELSE DO
            IF pos('UHOST-', TRANSLATE( hostname.i ))=0 & SWAPHIP = 0 THEN '@PING '||TRANSLATE( word(translate(hostname.i,' ','.'),1) )||' 1 1 2>&1|RXQUEUE'
            IF pos('UHOST-', TRANSLATE( hostname.i ))=0 & SWAPHIP = 1 THEN '@PING '||ipnumber.i||' 1 1 2>&1|RXQUEUE'
            DO WHILE QUEUED() > 0; PARSE PULL; END;
	    IF 0 < ARG() & ipnumber.0 = 1 & pos('UHOST-', TRANSLATE( hostname.i ))>0 THEN
		DO
			'@PING '||TRANSLATE( ipnumber.i )||' 1 1 2>&1|RXQUEUE'
            		 DO WHILE QUEUED() > 0; PARSE PULL; END;
			'@HOST' ipnumber.1 '2>&1|RXQUEUE'
			'@ARP '||TRANSLATE( ipnumber.i )||' 2>&1|RXQUEUE'
		END
	    ELSE
		IF SWAPHIP = 1 THEN '@ARP '||ipnumber.i||' 2>&1|RXQUEUE' 
		ELSE '@ARP '||TRANSLATE( word(translate(hostname.i,' ','.'),1) )||' 2>&1|RXQUEUE'
        END
        DO WHILE QUEUED() > 0
 	    QLine = LINEIN( 'QUEUE:' )
	    IF 0 < ARG() & ipnumber.0 = 1 & pos('=',QLine) > 0 THEN PARSE VALUE QLine WITH ipnumber.i' = 'hostname.i  
            PARSE VALUE SUBWORD( QLine, 2 + ( ipnumber.i = last ) ) WITH machex +17 ipnum .
	    IF lastpos(')',machex) > 0 THEN machex = substr(machex,1,lastpos(')',machex))
	    machex = translate(strip(machex),'0',' ')
	    IF lastpos(':',machex) = length(machex)-1 THEN machex = substr(machex,1,lastpos(':',machex))||'0'||substr(machex,lastpos(':',machex)+1)
	    IF pos('.',hostname.i) = 0 & isIP( ipnum ) & hostname.0 = 1 THEN   	
		DO
			TempIP = ipnum
			TempHName = hostname.i
			IF TempHName = '' THEN CALL SockGH
			ELSE Call missingdomain
			hostname.i = TempHName
 		END
	    IF ( STRIP( TRANSLATE( strip(machex) ) ) = 'RESOLVE)' | STRIP( TRANSLATE( strip(machex) ) ) = '(INCOMPLETE)' | IsMAC( strip(machex) ) | IsIP(Strip(translate(machex,'  ','()'))) ) & ( isIP( ipnum ) | ( ipnumber.i = last ) ) THEN
            DO
                pass = 0
                hostname.i = STRIP( hostname.i )
                IF ipnumber.i = last THEN
                DO
                    IF DATATYPE( STRIP( machex ), 'X' ) THEN
                    DO
                        ipnum = ipnumber.i
                        machex = TRANSLATE( 'ab:cd:ef:gh:ij:kl', STRIP( machex ), XRANGE( 'a', 'l' ) )
                        pass = 1
                    END
                END
                ELSE pass = 1
                IF pass THEN
                DO
                    j = j + 1
                    IF isStem THEN
                        INTERPRET stemval||".j.name = hostname.i;"||stemval||".j.macaddr = TRANSLATE( STRIP( machex ), '0', ' ' );"||stemval||".j.ipaddr = ipnum;"||stemval||".0 = j; retval = j"
                    ELSE
                        retval = retval||hostname.i||' '||TRANSLATE( STRIP( machex ), '0', ' ' )||' '||ipnum||D2C(13)||D2C(10)
                END
            END		    
        END
    END

RETURN retval
    
IsMAC: PROCEDURE
RETURN STRIP( TRANSLATE( word(ARG(1),1), '', XRANGE( '0', '9' )||XRANGE( 'a', 'f' )||':' ) ) = '' & word(ARG(1),1) <> 0 & ( LENGTH( word(ARG(1),1) ) = 12 | LENGTH( word(ARG(1),1) ) = 17 )
    
IsIP: PROCEDURE
    IF words(ARG())=2 THEN ipnum = word(ARG(1),2) 	
    ELSE ipnum = word(ARG(1),1)
    IF length(ipnum)<7 THEN RETURN 0 
    DO i = 1 TO 4
        PARSE VALUE ipnum WITH n '.' ipnum
        IF \DATATYPE( n, 'W' ) THEN RETURN 0
        IF n < ( 0 + ( i = 1 | i = 4 ) ) THEN RETURN 0
        IF 255 < n THEN RETURN 0
    END
RETURN 1

SockGH:	
	SCT = SockSocket( 'AF_INET', 'SOCK_DGRAM', 'IPPROTO_UDP' );
	TerminalAddr.!addr = TempIp
	TerminalAddr.!name = ''
	Call SockGetHostByAddr TerminalAddr.!addr, "TerminalAddr.!"
	SELECT
		WHEN (strip(TerminalAddr.!name)\='') & (strip(TempHName)='') THEN
			DO
				If VALUE( 'MVN',, 'OS2ENVIRONMENT' ) \= 1 THEN say "SockGetHost: "||TerminalAddr.!name||':' TerminalAddr.!addr;
				TempHName = TerminalAddr.!name
			End
  		WHEN (strip(TerminalAddr.!name)='') & (strip(TempHName)='') & (TerminalAddr.!addr\='') THEN
 				TempHName = 'Uhost-'||Right(word(translate(TerminalAddr.!addr,' ','.'),words(translate(TerminalAddr.!addr,' ','.'))),3,'0')
		OTHERWISE NOP
	END
  	IF pos('.',TempHName) = 0 & TerminalAddr.!addr \= '' THEN CALL MissingDomain
RETURN

Anetstat:
	IF ipnumber.lcounter='' & hostname.i\='' THEN '@PING '||TRANSLATE( word(translate(hostname.i,' ','.'),1) )||' 1 1 2>&1|RXQUEUE'
	DO WHILE QUEUED() > 0; PARSE PULL; END;
	'@NETSTAT -a 2>&1|RXQUEUE'
	DO queued()
		Parse pull locsock
			If wordpos('addr',locsock)=1 & wordpos('127.0.0.1',locsock)=0 THEN
				DO  		
					ipnumber.lcounter=word(locsock,2)
					leave
				END
	END
	DO WHILE QUEUED() > 0; PARSE PULL; END;
RETURN 

MissingDomain:
	IF TempHName = '' & TempIp = '' then return 
	/* IF word(UseIntrfAsDomain,2) \= '' THEN TempHName = TempHName||'.'||word(UseIntrfAsDomain,2) */
	ELSE
		DO
		'@NETSTAT -r 2>&1|RXQUEUE'
		'@NETSTAT -p 2>&1|RXQUEUE'
			DO queued()
				Parse pull routtbl
				IF wordpos(TempIp,routtbl)=2 & words(routtbl)>5 THEN
					DO
						Intrfnum=Strip(translate(word(routtbl,6),' ','abcdefghijklmnopqrstuvwxyz'))
						If Intrfnum='' then Intrfnum=0
						UseIntrfAsDomain=Intrfnum||' '||Strip(translate(word(routtbl,6),' ','0123456789'))											
						TempHName = TempHName||'.'||word(UseIntrfAsDomain,2)
						If word(routtbl,5)='UC' THEN
							DO
								ThisHIP = TempIp
								If wordpos(word(routtbl,1),Intf_NETX) = 0 THEN Intf_NETX=Strip(Intf_NETX||' '||word(routtbl,1)||' '||Intrfnum)
							END
						If pos('UG',word(routtbl,5)) > 0 THEN GWip = TempIp
						leave
					END
				IF wordpos(TempIp,routtbl)=3 & words(routtbl)=4 THEN
					DO
						Intrfnum=Strip(translate(word(routtbl,1),' ','abcdefghijklmnopqrstuvwxyz'))
						If Intrfnum='' then Intrfnum=0
						UseIntrfAsDomain=Intrfnum||' '||Strip(translate(word(routtbl,1),' ','0123456789'))											
						TempHName = TempHName||'.'||word(UseIntrfAsDomain,2)
						If wordpos(substr(TempIp,1,lastpos('.',TempIp)-1),Intf_NETX) = 0 THEN Intf_NETX=Strip(Intf_NETX||' '||substr(TempIp,1,lastpos('.',TempIp)-1)||' '||Intrfnum)
						leave
					END
			END
			DO WHILE QUEUED() > 0; PARSE PULL; END;
		END
RETURN

LANKO:
IF LANPC = 1 & If VALUE( 'MVN',, 'OS2ENVIRONMENT' ) \= 1 THEN then SAY "Disconnected cable ?" 
If VALUE( 'MVN',, 'OS2ENVIRONMENT' ) \= 1 THEN SAY "Network wasn't UP for a while !"  
EXIT 28

MAC_WRITE:
If stream(macfile,'c','query exists')\='' THEN '@del "'||macfile||'" /N'
Call STREAM macfile,'c','open write'
Rrc=CHAROUT(macfile,,1)
Rrc=CHAROUT(macfile,macfilestring,1)
Call STREAM macfile,'c','close'
'SET FLC=0'
RETURN

MAC_READ:
k = 0
Do_update = 0
If stream(macfile,'c','query exists')\='' THEN 
	DO UNTIL LINES(macfile)<>1
		k = k+1
		cfg.k = LINEIN(macfile)
		If datatype(word(cfg.k,1),'N') = 0 THEN
			DO
				Do_update = 1 
				cfg.k = date(B)||' '||cfg.k
			END
		macfilestring=Strip(macfilestring||cfg.k||x2c("0D0A"))		 
	END
cfg.0 = k
Call STREAM macfile,'c','close'
If Do_update = 1 Then Call MAC_WRITE
macfilestring = ''
RETURN	 